==> Sculptor-darwin-arm64-0.2.26.zip: Sculptor.app/Contents/Resources/sculptor_backend/_internal/sculptor/agents/default/claude_code_sdk/agent_wrapper.py <==
"""Uses the Anthropic Claude Code SDK to run a Claude Code agent.

Particularly, headless mode: https://docs.anthropic.com/en/docs/claude-code/sdk/sdk-headless
"""

import json
import shlex
from pathlib import Path
from queue import Queue
from typing import assert_never

from loguru import logger
from pydantic import PrivateAttr

from imbue_core.agents.data_types.ids import TaskID
from imbue_core.sculptor.state.messages import ChatInputUserMessage
from imbue_core.sculptor.state.messages import Message
from imbue_core.secrets_utils import Secret
from sculptor.agents.default.agent_wrapper import DefaultAgentWrapper
from sculptor.agents.default.claude_code_sdk.artifact_creation import get_file_artifact_messages
from sculptor.agents.default.claude_code_sdk.claude_process_manager import ClaudeProcessManager
from sculptor.agents.default.claude_code_sdk.config_utils import populate_claude_settings
from sculptor.agents.default.claude_code_sdk.utils import get_warning_message
from sculptor.agents.default.constants import DEFAULT_WAIT_TIMEOUT
from sculptor.agents.default.constants import GITLAB_PROJECT_URL_STATE_FILE
from sculptor.agents.default.constants import GITLAB_TOKEN_STATE_FILE
from sculptor.agents.default.constants import REMOVED_MESSAGE_IDS_STATE_FILE
from sculptor.constants import PROXY_CACHE_PATH
from sculptor.interfaces.agents.agent import ClaudeCodeSDKAgentConfig
from sculptor.interfaces.agents.agent import CommandInputUserMessage
from sculptor.interfaces.agents.agent import CompactTaskUserMessage
from sculptor.interfaces.agents.agent import ForkAgentSystemMessage
from sculptor.interfaces.agents.agent import GitCommitAndPushUserMessage
from sculptor.interfaces.agents.agent import GitPullUserMessage
from sculptor.interfaces.agents.agent import InterruptProcessUserMessage
from sculptor.interfaces.agents.agent import LocalSyncDisabledMessage
from sculptor.interfaces.agents.agent import LocalSyncSetupAndEnabledMessage
from sculptor.interfaces.agents.agent import LocalSyncSetupProgressMessage
from sculptor.interfaces.agents.agent import LocalSyncSetupStartedMessage
from sculptor.interfaces.agents.agent import LocalSyncUpdateCompletedMessage
from sculptor.interfaces.agents.agent import LocalSyncUpdatePausedMessage
from sculptor.interfaces.agents.agent import LocalSyncUpdatePendingMessage
from sculptor.interfaces.agents.agent import ManualSyncMergeIntoAgentAttemptedMessage
from sculptor.interfaces.agents.agent import MessageFeedbackUserMessage
from sculptor.interfaces.agents.agent import RemoveQueuedMessageAgentMessage
from sculptor.interfaces.agents.agent import RemoveQueuedMessageUserMessage
from sculptor.interfaces.agents.agent import RequestSkippedAgentMessage
from sculptor.interfaces.agents.agent import ResumeAgentResponseRunnerMessage
from sculptor.interfaces.agents.agent import SetProjectConfigurationDataUserMessage
from sculptor.interfaces.agents.agent import SetUserConfigurationDataUserMessage
from sculptor.interfaces.agents.agent import StopAgentUserMessage
from sculptor.interfaces.agents.artifacts import ArtifactType
from sculptor.interfaces.agents.constants import AGENT_EXIT_CODE_CLEAN_SHUTDOWN_ON_INTERRUPT
from sculptor.interfaces.agents.constants import AGENT_EXIT_CODE_SHUTDOWN_DUE_TO_EXCEPTION
from sculptor.interfaces.environments.base import Environment
from sculptor.services.config_service.data_types import AWSBedrockApiKey
from sculptor.services.config_service.data_types import AnthropicApiKey
from sculptor.services.config_service.data_types import AnthropicCredentials
from sculptor.services.config_service.data_types import ClaudeOauthCredentials
from sculptor.tasks.handlers.run_agent.errors import GitCommandFailure
from sculptor.tasks.handlers.run_agent.git import run_git_command_in_environment


class ClaudeCodeSDKAgent(DefaultAgentWrapper):
    config: ClaudeCodeSDKAgentConfig
    in_testing: bool = False
    snapshot_path: Path | None = None
    _claude_process_manager: ClaudeProcessManager | None = PrivateAttr(default=None)

    def _terminate(self, force_kill_seconds: float = 5.0) -> None:
        assert self._claude_process_manager is not None, "Claude process manager must be set"
        self._claude_process_manager.stop(force_kill_seconds, is_waiting=False)

    def poll(self) -> int | None:
        assert self._claude_process_manager is not None, "Claude process manager must be set"
        if self._claude_process_manager.get_exception_if_exists() is not None:
            self._exit_code = AGENT_EXIT_CODE_SHUTDOWN_DUE_TO_EXCEPTION
        return super().poll()

    def wait(self, timeout: float) -> int:
        assert self._claude_process_manager is not None, "Claude process manager must be set"
        self._claude_process_manager.stop(timeout, is_waiting=True)

        assert self._exit_code is not None, (
            "The wait method will only ever terminate if the agent is stopped or if there is an exception"
        )
        return self._exit_code

    def _start(self) -> None:
        # Initialize the Claude process manager
        self._claude_process_manager = ClaudeProcessManager(
            environment=self.environment,
            in_testing=self.in_testing,
            secrets=self._secrets,
            task_id=self.task_id,
            output_message_queue=self._output_messages,
            handle_user_message_callback=self._handle_user_message,
            system_prompt=self.system_prompt,
            source_branch=self.source_branch,
            task_branch=self.task_branch,
        )

    def _update_credentials(self):
        # make sure we've updated the credentials
        anthropic_credentials = self._get_anthropic_credentials()
        self._load_credentials(anthropic_credentials=anthropic_credentials)

    def _push_message(self, message: Message) -> None:
        # Only emit PostHog events for actual UserMessageUnion types
        # Some callers pass non-UserMessageUnion types with pyre-ignore comments
        # (e.g., StartLocalSyncRunnerMessage in v1.py:336)
        # Check the runtime type against UserMessageUnion to be defensive

        # TODO: this is missing 2 cases! StopCheckUserMessage and RestartCheckUserMessage
        match message:
            case RemoveQueuedMessageUserMessage():
                with self._handle_user_message(message):
                    self._removed_message_ids.add(message.target_message_id.suffix)
                    self.environment.write_file(
                        str(self.environment.get_state_path() / REMOVED_MESSAGE_IDS_STATE_FILE),
                        json.dumps(list(self._removed_message_ids)),
                    )
                    logger.info("Removed message id: {}", message.target_message_id)
                    self._output_messages.put(
                        RemoveQueuedMessageAgentMessage(removed_message_id=message.target_message_id)
                    )
            case CommandInputUserMessage() | ChatInputUserMessage() | ResumeAgentResponseRunnerMessage():
                if message.message_id.suffix in self._removed_message_ids:
                    logger.info("Skipping message {} as it has been removed", message.message_id)
                    self._output_messages.put(
                        # TODO: pyre doesn't understand pydantic
                        RequestSkippedAgentMessage(request_id=message.message_id)  # pyre-fixme[28]
                    )
                    return
                assert self._claude_process_manager is not None, "Claude process manager must be set"
                self._update_credentials()
                self._claude_process_manager.process_input_message(message)  # pyre-ignore[16]
            case LocalSyncUpdateCompletedMessage() | ManualSyncMergeIntoAgentAttemptedMessage():
                logger.info("Received local sync update message, updating artifacts")
                messages_to_send = get_file_artifact_messages(
                    artifact_name=ArtifactType.DIFF,
                    environment=self.environment,
                    source_branch=self.source_branch,
                    task_id=self.task_id,
                )
                for artifact_message in messages_to_send:
                    self._output_messages.put(artifact_message)
            case (
                LocalSyncSetupStartedMessage()
                | LocalSyncSetupProgressMessage()
                | LocalSyncSetupAndEnabledMessage()
                | LocalSyncUpdatePendingMessage()
                | LocalSyncUpdatePausedMessage()
                | LocalSyncDisabledMessage()
            ):
                pass
            # TODO: eventually just make this GitCommitUserMessage
            case GitCommitAndPushUserMessage():
                with self._handle_user_message(message):
                    commit_message = shlex.quote(message.commit_message)
                    task_branch = shlex.quote(self.task_branch)
                    commit_and_push_command_string = f"if [ \"$(git branch --show-current)\" != {task_branch} ]; then echo 'Error: Current branch is not {task_branch}'; exit 1; fi && git add . && git commit -m {commit_message} --trailer 'Co-authored-by: Sculptor <sculptor@imbue.com>'"
                    # when settings.IS_NEW_MANUAL_SYNC_ENABLED is true, we do not want to push
                    if message.is_pushing:
                        commit_and_push_command_string += " && git push sculptor"
                    on_git_user_message(
                        environment=self.environment,
                        command=["bash", "-c", commit_and_push_command_string],
                        source_branch=self.source_branch,
                        output_message_queue=self._output_messages,
                        task_id=self.task_id,
                    )
            case GitPullUserMessage():
                with self._handle_user_message(message):
                    on_git_user_message(
                        environment=self.environment,
                        command=["git", "pull"],
                        source_branch=self.source_branch,
                        output_message_queue=self._output_messages,
                        task_id=self.task_id,
                    )
            # FIXME: make an error message for local sync
            case CompactTaskUserMessage():
                assert self._claude_process_manager is not None, "Claude process manager must be set"
                self._update_credentials()
                self._claude_process_manager.process_input_message(message)
            case InterruptProcessUserMessage():
                assert self._claude_process_manager is not None, "Claude process manager must be set"
                self._claude_process_manager.interrupt_current_message(message)
            case StopAgentUserMessage():
                logger.info("Stopping agent")
                with self._handle_user_message(message):
                    self.terminate(DEFAULT_WAIT_TIMEOUT)
                    self._exit_code = AGENT_EXIT_CODE_CLEAN_SHUTDOWN_ON_INTERRUPT
                # # it doesn't make sense to make markers for starting and stopping this message
                # # since the semantics are that, once stopping, we no longer get the RequestSuccessAgentMessage messages
                # # so we very specifically do NOT call `self._handle_user_message(message)` here
                # self._is_stopping = True
                # self.terminate(DEFAULT_WAIT_TIMEOUT)
                # self._exit_code = AGENT_EXIT_CODE_CLEAN_SHUTDOWN_ON_INTERRUPT
                logger.info("Finished stopping agent")
            case SetUserConfigurationDataUserMessage():
                logger.info("User configuration message received")
                anthropic_credentials = message.anthropic_credentials
                if anthropic_credentials is not None:
                    self._load_credentials(anthropic_credentials)
            case SetProjectConfigurationDataUserMessage():
                logger.info("Project configuration message received")
                self.environment.write_file(
                    str(self.environment.get_state_path() / GITLAB_TOKEN_STATE_FILE), message.gitlab_token
                )
                self.environment.write_file(
                    str(self.environment.get_state_path() / GITLAB_PROJECT_URL_STATE_FILE), message.gitlab_url
                )
            case MessageFeedbackUserMessage():
                logger.info("Message feedback received for message {}", message.feedback_message_id)
                pass
            case ForkAgentSystemMessage():
                pass
            case _ as unreachable:
                # pyre-fixme[6]: The type of message should be changed to an appropriate union type instead of a base type
                assert_never(unreachable)

    def _initialize_for_testing(self, anthropic_credentials: AnthropicCredentials) -> None:
        assert self.in_testing, "setup_testing should only be called when in testing"
        assert "ANTHROPIC_API_KEY" not in self._secrets
        # This setup for testing is slightly tricky.
        # We inject a (valid) Anthropic API key for testing, but we don't want Claude Code to actually use it;
        # instead, we want the proxy to use it, and Claude Code to use the proxy.
        #
        # 1. We extract the actual Anthropic credentials,
        #    which must be an API key because the proxy is only set up to accept that.
        #
        # 2. We re-assign anthropic_credentials to a fake credential so that Claude Code can't see the real one.
        #    This isn't strictly necessary, but it makes sure that Claude Code can't access Anthropic API directly,
        #    just in case the ANTHROPIC_BASE_URL override is somehow not set up properly.
        #
        # TODO: Clean this up so that it's less confusing.
        assert isinstance(anthropic_credentials, AnthropicApiKey)
        anthropic_api_key = anthropic_credentials.anthropic_api_key
        anthropic_credentials = AnthropicApiKey(
            anthropic_api_key=Secret("sk-ant-hidden-for-testing"), generated_from_oauth=False
        )
        proxy_secrets = dict(self._secrets)

        snapshot_path = self.snapshot_path
        if snapshot_path is not None:
            proxy_secrets["SNAPSHOT_PATH"] = PROXY_CACHE_PATH
            try:
                self.environment.copy_from_local(snapshot_path, PROXY_CACHE_PATH, recursive=True)
            except FileNotFoundError:
                logger.error("Missing snapshot file {} for test", snapshot_path)
                raise
        else:
            proxy_secrets["ANTHROPIC_API_KEY"] = anthropic_api_key
        logger.info("proxy secrets: {}", proxy_secrets)

        # Thad: I do not completely understand the reason, but this just does not work if you run the proxy as root.
        # Integration tests just stall trying to talk to it.
        self.environment.run_process_in_background(
            ["/imbue/.venv/bin/python", "/imbue/claude_code_proxy.py"], secrets=proxy_secrets, run_as_root=False
        )

        if self._secrets.get("ANTHROPIC_BASE_URL"):
            raise Exception(
                "In testing but ANTHROPIC_BASE_URL was set, this should not happen. The tests override this variable to implement LLM caching."
            )
        logger.debug("Forcing an override of ANTHROPIC_BASE_URL to localhost for testing")
        self._secrets["ANTHROPIC_BASE_URL"] = "http://localhost:8082"

    def _load_credentials(self, anthropic_credentials: AnthropicCredentials | None) -> None:
        """This loads the anthropic creds into the container AND the ClaudeProcessManager."""
        assert anthropic_credentials is not None, "Anthropic credentials must be set"
        match anthropic_credentials:
            case AnthropicApiKey(anthropic_api_key=anthropic_api_key):
                self._secrets["ANTHROPIC_API_KEY"] = anthropic_api_key
                self._secrets.pop("IMBUE_ANTHROPIC_AUTH_TOKEN", None)
            case AWSBedrockApiKey(bedrock_api_key=bedrock_api_key):
                self._secrets["AWS_BEARER_TOKEN_BEDROCK"] = bedrock_api_key
                self._secrets["CLAUDE_CODE_USE_BEDROCK"] = "1"
                self._secrets.pop("IMBUE_ANTHROPIC_AUTH_TOKEN", None)
            case ClaudeOauthCredentials():
                # Claude Code prioritizes ANTHROPIC_API_KEY over OAuth credentials,
                # so we have to remove it.
                self._secrets.pop("ANTHROPIC_API_KEY", None)
                self._secrets.pop("AWS_BEARER_TOKEN_BEDROCK", None)
                # Not used by Claude Code itself, but by imbue_verify.
                # Search for IMBUE_ANTHROPIC_AUTH_TOKEN in imbue_core to see where it's used.
                self._secrets["IMBUE_ANTHROPIC_AUTH_TOKEN"] = anthropic_credentials.access_token
            case _ as unreachable:
                # pyre doesn't understand the matching here
                assert_never(unreachable)  # pyre-ignore[6]
        populate_claude_settings(self.environment, anthropic_credentials)


def on_git_user_message(
    environment: Environment,
    command: list[str],
    source_branch: str,
    output_message_queue: Queue[Message],
    task_id: TaskID,
) -> None:
    try:
        logger.info("Running git command: {}", " ".join(command))
        run_git_command_in_environment(
            environment=environment,
            command=command,
            secrets={},
            cwd=str(environment.get_workspace_path()),
            is_retry_safe=True,
            timeout=30.0,
        )
    except GitCommandFailure as e:
        output_message_queue.put(
            get_warning_message(
                f"Failed to run git command {command} - stderr: {e.stderr}",
                e,
                task_id,
            )
        )
    logger.info("Received git user message, updating artifacts")
    messages_to_send = get_file_artifact_messages(
        artifact_name=ArtifactType.DIFF,
        environment=environment,
        source_branch=source_branch,
        task_id=task_id,
    )
    for artifact_message in messages_to_send:
        output_message_queue.put(artifact_message)

==> Sculptor-darwin-arm64-0.2.29.zip: Sculptor.app/Contents/Resources/sculptor_backend/_internal/sculptor/agents/default/claude_code_sdk/agent_wrapper.py <==
"""Uses the Anthropic Claude Code SDK to run a Claude Code agent.

Particularly, headless mode: https://docs.anthropic.com/en/docs/claude-code/sdk/sdk-headless
"""

from pathlib import Path
from typing import assert_never

from loguru import logger
from pydantic import PrivateAttr

from imbue_core.sculptor.state.messages import ChatInputUserMessage
from imbue_core.sculptor.state.messages import Message
from imbue_core.secrets_utils import Secret
from sculptor.agents.default.agent_wrapper import DefaultAgentWrapper
from sculptor.agents.default.claude_code_sdk.config_utils import populate_claude_settings
from sculptor.agents.default.claude_code_sdk.process_manager import ClaudeProcessManager
from sculptor.constants import PROXY_CACHE_PATH
from sculptor.interfaces.agents.agent import ClaudeCodeSDKAgentConfig
from sculptor.interfaces.agents.agent import CommandInputUserMessage
from sculptor.interfaces.agents.agent import CompactTaskUserMessage
from sculptor.interfaces.agents.agent import InterruptProcessUserMessage
from sculptor.interfaces.agents.agent import RequestSkippedAgentMessage
from sculptor.interfaces.agents.agent import ResumeAgentResponseRunnerMessage
from sculptor.interfaces.agents.constants import AGENT_EXIT_CODE_SHUTDOWN_DUE_TO_EXCEPTION
from sculptor.services.config_service.data_types import AWSBedrockApiKey
from sculptor.services.config_service.data_types import AnthropicApiKey
from sculptor.services.config_service.data_types import AnthropicCredentials
from sculptor.services.config_service.data_types import ClaudeOauthCredentials


class ClaudeCodeSDKAgent(DefaultAgentWrapper):
    config: ClaudeCodeSDKAgentConfig
    in_testing: bool = False
    snapshot_path: Path | None = None
    _claude_process_manager: ClaudeProcessManager | None = PrivateAttr(default=None)

    def _terminate(self, force_kill_seconds: float = 5.0) -> None:
        assert self._claude_process_manager is not None, "Claude process manager must be set"
        self._claude_process_manager.stop(force_kill_seconds, is_waiting=False)

    def poll(self) -> int | None:
        assert self._claude_process_manager is not None, "Claude process manager must be set"
        if self._claude_process_manager.get_exception_if_exists() is not None:
            self._exit_code = AGENT_EXIT_CODE_SHUTDOWN_DUE_TO_EXCEPTION
        return super().poll()

    def wait(self, timeout: float) -> int:
        assert self._claude_process_manager is not None, "Claude process manager must be set"
        self._claude_process_manager.stop(timeout, is_waiting=True)

        assert self._exit_code is not None, (
            "The wait method will only ever terminate if the agent is stopped or if there is an exception"
        )
        return self._exit_code

    def _start(self) -> None:
        # Initialize the Claude process manager
        self._claude_process_manager = ClaudeProcessManager(
            environment=self.environment,
            in_testing=self.in_testing,
            secrets=self._secrets,
            task_id=self.task_id,
            output_message_queue=self._output_messages,
            handle_user_message_callback=self._handle_user_message,
            system_prompt=self.system_prompt,
            source_branch=self.source_branch,
            task_branch=self.task_branch,
        )

    def _update_credentials(self):
        # make sure we've updated the credentials
        anthropic_credentials = self._get_anthropic_credentials()
        self._load_credentials(anthropic_credentials=anthropic_credentials)

    def _push_message(self, message: Message) -> bool:
        match message:
            case CommandInputUserMessage() | ChatInputUserMessage() | ResumeAgentResponseRunnerMessage():
                if message.message_id.suffix in self._removed_message_ids:
                    logger.info("Skipping message {} as it has been removed", message.message_id)
                    self._output_messages.put(
                        # TODO: pyre doesn't understand pydantic
                        RequestSkippedAgentMessage(request_id=message.message_id)  # pyre-fixme[28]
                    )
                else:
                    assert self._claude_process_manager is not None, "Claude process manager must be set"
                    self._update_credentials()
                    self._claude_process_manager.process_input_message(message=message)  # pyre-ignore[16]
            case CompactTaskUserMessage():
                assert self._claude_process_manager is not None, "Claude process manager must be set"
                self._update_credentials()
                self._claude_process_manager.process_compact_message(message=message)  # pyre-ignore[16]
            case InterruptProcessUserMessage():
                assert self._claude_process_manager is not None, "Claude process manager must be set"
                self._claude_process_manager.interrupt_current_message(message=message)
            case _:
                return False
        return True

    def _initialize_for_testing(self, anthropic_credentials: AnthropicCredentials) -> None:
        assert self.in_testing, "setup_testing should only be called when in testing"
        assert "ANTHROPIC_API_KEY" not in self._secrets
        # This setup for testing is slightly tricky.
        # We inject a (valid) Anthropic API key for testing, but we don't want Claude Code to actually use it;
        # instead, we want the proxy to use it, and Claude Code to use the proxy.
        #
        # 1. We extract the actual Anthropic credentials,
        #    which must be an API key because the proxy is only set up to accept that.
        #
        # 2. We re-assign anthropic_credentials to a fake credential so that Claude Code can't see the real one.
        #    This isn't strictly necessary, but it makes sure that Claude Code can't access Anthropic API directly,
        #    just in case the ANTHROPIC_BASE_URL override is somehow not set up properly.
        #
        # TODO: Clean this up so that it's less confusing.
        assert isinstance(anthropic_credentials, AnthropicApiKey)
        anthropic_api_key = anthropic_credentials.anthropic_api_key
        anthropic_credentials = AnthropicApiKey(
            anthropic_api_key=Secret("sk-ant-hidden-for-testing"), generated_from_oauth=False
        )
        proxy_secrets = dict(self._secrets)

        snapshot_path = self.snapshot_path
        if snapshot_path is not None:
            proxy_secrets["SNAPSHOT_PATH"] = PROXY_CACHE_PATH
            try:
                self.environment.copy_from_local(snapshot_path, PROXY_CACHE_PATH, recursive=True)
            except FileNotFoundError:
                logger.error("Missing snapshot file {} for test", snapshot_path)
                raise
        else:
            proxy_secrets["ANTHROPIC_API_KEY"] = anthropic_api_key
        logger.info("proxy secrets: {}", proxy_secrets)

        # Thad: I do not completely understand the reason, but this just does not work if you run the proxy as root.
        # Integration tests just stall trying to talk to it.
        self.environment.run_process_in_background(
            ["/imbue/.venv/bin/python", "/imbue/claude_code_proxy.py"], secrets=proxy_secrets, run_as_root=False
        )

        if self._secrets.get("ANTHROPIC_BASE_URL"):
            raise Exception(
                "In testing but ANTHROPIC_BASE_URL was set, this should not happen. The tests override this variable to implement LLM caching."
            )
        logger.debug("Forcing an override of ANTHROPIC_BASE_URL to localhost for testing")
        self._secrets["ANTHROPIC_BASE_URL"] = "http://localhost:8082"

    def _load_credentials(self, anthropic_credentials: AnthropicCredentials | None) -> None:
        """This loads the anthropic creds into the container AND the ClaudeProcessManager."""
        assert anthropic_credentials is not None, "Anthropic credentials must be set"
        match anthropic_credentials:
            case AnthropicApiKey(anthropic_api_key=anthropic_api_key):
                self._secrets["ANTHROPIC_API_KEY"] = anthropic_api_key
                self._secrets.pop("IMBUE_ANTHROPIC_AUTH_TOKEN", None)
            case AWSBedrockApiKey(bedrock_api_key=bedrock_api_key):
                self._secrets["AWS_BEARER_TOKEN_BEDROCK"] = bedrock_api_key
                self._secrets["CLAUDE_CODE_USE_BEDROCK"] = "1"
                self._secrets.pop("IMBUE_ANTHROPIC_AUTH_TOKEN", None)
            case ClaudeOauthCredentials():
                # Claude Code prioritizes ANTHROPIC_API_KEY over OAuth credentials,
                # so we have to remove it.
                self._secrets.pop("ANTHROPIC_API_KEY", None)
                self._secrets.pop("AWS_BEARER_TOKEN_BEDROCK", None)
                # Not used by Claude Code itself, but by imbue_verify.
                # Search for IMBUE_ANTHROPIC_AUTH_TOKEN in imbue_core to see where it's used.
                self._secrets["IMBUE_ANTHROPIC_AUTH_TOKEN"] = anthropic_credentials.access_token
            case _ as unreachable:
                # pyre doesn't understand the matching here
                assert_never(unreachable)  # pyre-ignore[6]
        populate_claude_settings(self.environment, anthropic_credentials)
